/*
 * Copyright (c) 2010, 2014 TradertoolsFX
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:

 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.

 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#property copyright "Copyright (c) 2010, 2014 TradertoolsFX"
#property link      "http://www.tradertools-fx.com"

struct ObjectProperties {
    int width;
    int style;
    color color_;

    ObjectProperties(void) : width(0), style(0), color_(0)
    {}

    ObjectProperties(int objwidth, int objstyle, int objcolor) :
        width(objwidth),
        style(objstyle),
        color_(objcolor)
    {}

    ObjectProperties(const ObjectProperties &other)
    {
        copy(other);
    }

    void copy(const ObjectProperties &other) {
        width = other.width;
        style = other.style;
        color_ = other.color_;
    }
    void operator=(const ObjectProperties &other) { copy(other); }
};

struct ZoneSettings {
    ObjectProperties props;
    bool do_draw;
    bool do_fill;
    double range_pips;

    ZoneSettings(const ObjectProperties &objprops,
                 bool draw,
                 bool fill,
                 double range_in_pips) :
        props(objprops),
        do_draw(draw),
        do_fill(fill),
        range_pips(range_in_pips)
    {}

    ZoneSettings(const ZoneSettings &other) :
        props(other.props),
        do_draw(other.do_draw),
        do_fill(other.do_fill),
        range_pips(other.range_pips)
    {}
};

class LineDrawer {
public:
    LineDrawer(const ObjectProperties &line_settings,
               const ZoneSettings &zone_settings,
               const string &object_prefix) :
        m_line(line_settings),
        m_zone(zone_settings),
        m_objprefix(object_prefix),
        m_objcount(0)

    {
        MathSrand(GetTickCount());
    }

    void set_line_price(double price)
    {
        m_line_price = NormalizeDouble(price, Digits);
    }

    void draw()
    {
        if (m_line_price <= 0) {
            Print("Warning! m_line_price not set in ", __FUNCTION__);
        }

        draw_hline(m_line_price);
        if (m_zone.do_draw) {
            draw_zone();
        }
    }
private:
    ObjectProperties m_line;
    ZoneSettings m_zone;
    string m_objprefix;
    int m_objcount;
    double m_line_price;

    void draw_hline(double price, bool is_zone=false)
    {
        ObjectProperties props;
        if (is_zone) {
            props = m_zone.props;
        } else {
            props = m_line;
        }

        string name = generate_object_name();
        if (ObjectCreate(name, OBJ_HLINE, 0, 0, price)) {
            ObjectSet(name, OBJPROP_COLOR, props.color_);
            ObjectSet(name, OBJPROP_STYLE, props.style);
            ObjectSet(name, OBJPROP_WIDTH, props.width);
        }
    }

    void draw_zone(void)
    {
        double price_distance_per_side = (m_zone.range_pips/2) * Point;
        double price_boundary[2];
        price_boundary[0] = m_line_price - price_distance_per_side;
        price_boundary[1] = m_line_price + price_distance_per_side;
        for (int i = 0; i < 2; i++) {
            draw_hline(price_boundary[i], true);
        }

        if (m_zone.do_fill) {
            string name = generate_object_name();
            if (ObjectCreate(name, OBJ_RECTANGLE, 0,
                             Time[Bars-1], price_boundary[0],
                             Time[0], price_boundary[1])) {
                ObjectSet(name, OBJPROP_COLOR, m_zone.props.color_);
                ObjectSet(name, OBJPROP_BACK, true);
            } else {
                Print(__FUNCTION__,
                      ": Failed to creae rectangle. Last error: ",
                      GetLastError());
            }
        }
    }

    string generate_object_name(void)
    {
        return StringConcatenate(m_objprefix, m_objcount++, '_', MathRand());
    }
};
